#####Simulations in Section 4.2#####
set.seed(123456)	#set.seed(123) for n=25, set.seed(1234) for n=50, set.seed(12345) for n=100, and set.seed(123456) for n=500 are used.
library(momentchi2)
library(fda)
number.rep <- 1000	#Number of replications
n <- 500	#Sample size
wid <- .001		#Accuracy
u_bound <- 2.8	#Upper bound of the domain
l_bound <- 0.2	#Lower bound of the domain
eta <- u_bound - l_bound	
z <- seq(l_bound, u_bound, wid)
number.z <- length(z)
number.basis <- 2*10	#Numbr of basis. This basis is used to generate an infinite-dimensional standard Gaussian variable.
root.lambda <- rep(2/3, number.basis)	#This is used to generate an infinite-dimensional standard Gaussian variable.
bspline_basis <- create.bspline.basis(c(l_bound, u_bound), nbasis=30, norder=4)	#This B-spline basis is for the calculation of eigenvalues. 

norm_L2_disc <- function(x){		#L2 norm
	x <- x^2
	integ <- wid*sum((1/2)*sum(x[1], x[number.z]), sum(x[2:(number.z -1)]))
	norm <- integ^(1/2)
	return(norm)
	}
est <- function(Y_mat, X_var, X_var_ast){			#The least square estimate
	beta_disc <- solve(crossprod(X_var), crossprod(X_var, Y_mat))
	beta_ast_disc <- solve(crossprod(X_var_ast), crossprod(X_var_ast, Y_mat))
	clr_y_hat_disc <- X_var%*%beta_disc
	clr_y_hat_ast_disc <- X_var_ast%*%beta_ast_disc
	error_disc <- matrix(0, ncol=length(z), nrow=n)
	for(i in 1:n){
		error_disc[i, ] <- Y_mat[i, ] - clr_y_hat_disc[i, ]
	}
	error_ast_disc <- matrix(0, ncol=length(z), nrow=n)
	for(i in 1:n){
		error_ast_disc[i, ] <- Y_mat[i, ] - clr_y_hat_ast_disc[i, ]
	}
	norm_error <- numeric(n)
	for(i in 1:n){
		norm_error[i] <- norm_L2_disc(error_disc[i,])^2
	}
	sum_error <- sum(norm_error)
	norm_error_ast <- numeric(n)
	for(i in 1:n){
		norm_error_ast[i] <- norm_L2_disc(error_ast_disc[i, ])^2
	}
	sum_ast_error <- sum(norm_error_ast)
	test_lambda <- sum_ast_error - sum_error
	return(list(beta_disc=beta_disc, clr_y_hat_disc=clr_y_hat_disc, beta_ast_disc=beta_ast_disc, clr_y_hat_ast_disc=clr_y_hat_ast_disc, 
			error_disc=error_disc, error_ast_disc=error_ast_disc, sum_error=sum_error, sum_ast_error=sum_ast_error, test_lambda=test_lambda))
}
#####Samples generated under the hypothesis H^1_0#####
sum_asy_per <- rep(NA, number.rep)
for(k in 1:number.rep){
	H.norm <- as.list(rep(NA, n))		#Random variables drawn from H-valued Gaussian distribution. They are approximated by finite Fourier series.
	for(i in 1:n){
		v.norm <- rnorm(number.basis, 0, 1)
		F.basis <- create.fourier.basis(c(l_bound, u_bound), nbasis=(number.basis + 1), dropind=1)		#Fourier series
		eval.F.basis <- eval.basis(z, F.basis)
		coeff <- root.lambda * v.norm
		H.norm[[i]] <- eval.F.basis %*% coeff
		}
	rm.var <- runif(n, 0, 2)	
	Inv.var <- exp(rm.var)		#Inverse of the variance, which is also used as explanatory variable.
	density.lognormal <- matrix(NA, ncol=length(z), nrow=n)
	log.density <- matrix(NA, ncol=length(z), nrow=n)
	norm.const <- rep(NA, n)
	norm.const.log <- rep(NA, n)
	clr.lognormal <- matrix(NA, ncol=length(z), nrow=n)
	clr_y_matrix <- matrix(NA, ncol=length(z), nrow=n)
	for(i in 1:n){
		density.lognormal[i,] <- dlnorm(z, meanlog = 0, sdlog = sqrt(1/Inv.var[i]), log = FALSE)		#Density function of lognormal distribution.
		norm.const[i] <- wid*sum((1/2)*sum(density.lognormal[i,1], density.lognormal[i,number.z]), sum(density.lognormal[i,2:(number.z -1)]))	#Integration
		density.lognormal[i,] <- density.lognormal[i,]/norm.const[i]	#Normalization
		log.density[i,] <- log(density.lognormal[i,])
		norm.const.log[i] <- wid*sum((1/2)*sum(log.density[i,1], log.density[i,number.z]), sum(log.density[i,2:(number.z -1)]))
		clr.lognormal[i,] <- log.density[i,] - (1/eta)*norm.const.log[i]		#Clr transformation
		clr_y_matrix[i,] <- clr.lognormal[i,] + H.norm[[i]]
		}
###Constants###
	x_1 <- rep(1, n)
	x_2 <- Inv.var
	temp <- cbind(x_1, x_2)
	X_ex <- as.matrix(temp)
	temp_ast <- cbind(x_1)
	X_ex_ast <- matrix(temp_ast, ncol=1, nrow=n)
###Estimation###
	original_est_res <- est(clr_y_matrix, X_ex, X_ex_ast)
###Asymptotic result###
	error_z <- t(original_est_res$error_disc)		#T by n matrix is required. Transpose. 
	errorfd <- smooth.basis(z, error_z, bspline_basis)$fd
	eigensystem <- pca.fd(errorfd, nharm=10, centerfns=FALSE)		#Eigenvalues
	eig_lambda <- eigensystem$values
	cof_vec <- c(original_est_res$test_lambda)
	asy_per <- hbe(eig_lambda[1:20], cof_vec)	
	sum_asy_per[k] <- asy_per
	}
############################

#####Samples generated under the hypothesis H^2_0#####
cum_asy_per <- rep(NA, number.rep)
	for(k in 1:number.rep){
	H.norm <- as.list(rep(NA, n))	
	for(i in 1:n){
		v.norm <- rnorm(number.basis, 0, 1)
		F.basis <- create.fourier.basis(c(l_bound, u_bound), nbasis=(number.basis + 1), dropind=1)
		eval.F.basis <- eval.basis(z, F.basis)
		coeff <- root.lambda * v.norm
		H.norm[[i]] <- eval.F.basis %*% coeff
		}
	rm.var <- runif(n, 0, 2)
	Inv.var <- exp(rm.var)
	const.var <- exp(rep(1, n))
	density.lognormal <- matrix(NA, ncol=length(z), nrow=n)
	log.density <- matrix(NA, ncol=length(z), nrow=n)
	norm.const <- rep(NA, n)
	norm.const.log <- rep(NA, n)
	clr.lognormal <- matrix(NA, ncol=length(z), nrow=n)
	clr_y_matrix <- matrix(NA, ncol=length(z), nrow=n)
	for(i in 1:n){
		density.lognormal[i,] <- dlnorm(z, meanlog = 0, sdlog = sqrt(1/const.var[i]), log = FALSE)
		norm.const[i] <- wid*sum((1/2)*sum(density.lognormal[i,1], density.lognormal[i,number.z]), sum(density.lognormal[i,2:(number.z -1)]))
		density.lognormal[i,] <- density.lognormal[i,]/norm.const[i]
		log.density[i,] <- log(density.lognormal[i,])
		norm.const.log[i] <- wid*sum((1/2)*sum(log.density[i,1], log.density[i,number.z]), sum(log.density[i,2:(number.z -1)]))
		clr.lognormal[i,] <- log.density[i,] - (1/eta)*norm.const.log[i]
		clr_y_matrix[i,] <- clr.lognormal[i,] + H.norm[[i]]
		}
###Constants###
	x_1 <- rep(1, n)
	x_2 <- Inv.var
	temp <- cbind(x_1, x_2)
	X_ex <- as.matrix(temp)
	temp_ast <- cbind(x_1)
	X_ex_ast <- matrix(temp_ast, ncol=1, nrow=n)
###Estimation###
	original_est_res <- est(clr_y_matrix, X_ex, X_ex_ast)
###Asymptotic result###
	error_z <- t(original_est_res$error_disc)
	errorfd <- smooth.basis(z, error_z, bspline_basis)$fd
	eigensystem <- pca.fd(errorfd, nharm=10, centerfns=FALSE)
	eig_lambda <- eigensystem$values
	cof_vec <- c(original_est_res$test_lambda)
	asy_per <- hbe(eig_lambda[1:20], cof_vec)		 
	cum_asy_per[k] <- asy_per
	}
############################
length(sum_asy_per[sum_asy_per>.99]);length(sum_asy_per[sum_asy_per>.95]);length(sum_asy_per[sum_asy_per>.90])	#Number of Rejections under H^1_0
length(cum_asy_per[cum_asy_per>.99]);length(cum_asy_per[cum_asy_per>.95]);length(cum_asy_per[cum_asy_per>.90])	#Number of Rejections under H^2_0